@@ -25,7 +25,7 @@
 #include <libmpd/libmpd.h>
 
 #include "mpdclient.h"
-#include "preferences.h"
+#include "xfmpc.h"
 
 #define GET_PRIVATE(o) \
     (G_TYPE_INSTANCE_GET_PRIVATE ((o), XFMPC_TYPE_MPDCLIENT, XfmpcMpdclientPrivate))
@@ -62,7 +62,7 @@ static void             cb_status_changed                       (MpdObj *mi,
                                                                  gpointer user_data);
 static gchar *          _get_formatted_name                     (mpd_Song *song);
 static gchar *          _get_formatted_name_predefined          (mpd_Song *song,
-                                                                 XfmpcSongFormat song_format);
+                                                                 XfmpcPreferencesSongFormat song_format);
 static gchar *          _get_formatted_name_custom              (mpd_Song *song,
                                                                  const gchar *format);
 
@@ -98,6 +98,9 @@ struct _XfmpcMpdclientPrivate
   guint                     port;
   gchar                    *passwd;
   gboolean                  env_cached;
+  gboolean                  connecting;
+  guint                     connection_count;
+  GMutex                   *mutex;
 };
 
 
@@ -237,7 +240,12 @@ static void
 xfmpc_mpdclient_init (XfmpcMpdclient *mpdclient)
 {
   XfmpcMpdclientPrivate *priv = mpdclient->priv = GET_PRIVATE (mpdclient);
+
   priv->mi = mpd_new_default ();
+
+  if (!g_thread_supported ()) g_thread_init (NULL);
+  priv->mutex = g_mutex_new ();
+
   mpd_signal_connect_status_changed (priv->mi, (StatusChangedCallback)cb_status_changed, mpdclient);
 }
 
@@ -248,13 +256,15 @@ xfmpc_mpdclient_finalize (GObject *object)
   XfmpcMpdclientPrivate *priv = XFMPC_MPDCLIENT (mpdclient)->priv;
 
   mpd_free (priv->mi);
+  g_mutex_free (priv->mutex);
+
   (*G_OBJECT_CLASS (parent_class)->finalize) (object);
 }
 
 
 
 XfmpcMpdclient *
-xfmpc_mpdclient_get ()
+xfmpc_mpdclient_get_default ()
 {
   static XfmpcMpdclient *mpdclient = NULL;
 
@@ -273,7 +283,7 @@ static void
 xfmpc_mpdclient_initenv (XfmpcMpdclient *mpdclient)
 {
   XfmpcMpdclientPrivate *priv = XFMPC_MPDCLIENT (mpdclient)->priv;
-  XfmpcPreferences *preferences = xfmpc_preferences_get ();
+  XfmpcPreferences *preferences = xfmpc_preferences_get_default ();
   gboolean use_defaults;
 
   g_object_get (preferences, "mpd-use-defaults", &use_defaults, NULL);
@@ -319,23 +329,67 @@ xfmpc_mpdclient_initenv (XfmpcMpdclient *mpdclient)
     }
 }
 
-gboolean
-xfmpc_mpdclient_connect (XfmpcMpdclient *mpdclient)
+static gpointer
+xfmpc_mpdclient_connect_thread (XfmpcMpdclient *mpdclient)
 {
   XfmpcMpdclientPrivate *priv = XFMPC_MPDCLIENT (mpdclient)->priv;
-
-  if (xfmpc_mpdclient_is_connected (mpdclient))
-    return TRUE;
+  mpd_Connection *connection;
 
   xfmpc_mpdclient_initenv (mpdclient);
   mpd_set_hostname (priv->mi, priv->host);
   mpd_set_port (priv->mi, priv->port);
   mpd_set_password (priv->mi, (priv->passwd != NULL) ? priv->passwd : "");
+  mpd_set_connection_timeout (priv->mi, 5.0);
 
-  if (mpd_connect (priv->mi) != MPD_OK)
+  connection = mpd_newConnection (priv->host, priv->port, 5.0);
+  if (mpd_connect_real (priv->mi, connection))
+    {
+      if ((priv->connection_count++) == 1)
+        g_message (_("Failed to connect to MPD"));
+    }
+  else
+    {
+      priv->connection_count = 0;
+      g_message (_("Connected to MPD"));
+    }
+
+  priv->connecting = FALSE;
+
+  g_mutex_unlock (priv->mutex);
+
+  return NULL;
+}
+
+gboolean
+xfmpc_mpdclient_connect (XfmpcMpdclient *mpdclient)
+{
+  XfmpcMpdclientPrivate *priv = XFMPC_MPDCLIENT (mpdclient)->priv;
+
+  GThread *thread;
+
+  if (xfmpc_mpdclient_is_connected (mpdclient))
+    return TRUE;
+
+  /* return FALSE if a we are already trying to connect to mpd */
+  if (!g_mutex_trylock (priv->mutex))
+  {
+    g_warning ("Already connecting to mpd");
     return FALSE;
+  }
 
-  mpd_send_password (priv->mi);
+  priv->connecting = TRUE;
+
+  thread = g_thread_create ((GThreadFunc) xfmpc_mpdclient_connect_thread,
+          mpdclient, TRUE, NULL);
+
+  while (priv->connecting)
+  {
+    while (gtk_events_pending ())
+      gtk_main_iteration ();
+    g_usleep (200000);
+  }
+
+  g_thread_join (thread);
 
   g_signal_emit (mpdclient, signals[SIG_CONNECTED], 0);
 
@@ -744,7 +798,7 @@ xfmpc_mpdclient_playlist_read (XfmpcMpdclient *mpdclient,
       if (NULL != length)
         *length = g_strdup_printf ("%d:%02d", data->song->time / 60, data->song->time % 60);
       if (NULL != pos)
-      	*pos = data->song->pos;
+        *pos = data->song->pos;
       *id = data->song->id;
     }
 
@@ -782,46 +836,6 @@ xfmpc_mpdclient_playlist_get_total_time (XfmpcMpdclient *mpdclient)
   return seconds;
 }
 
-gboolean
-xfmpc_mpdclient_playlist_has_filename (XfmpcMpdclient *mpdclient,
-                                       const gchar *filename,
-                                       gboolean is_dir)
-{
-  XfmpcMpdclientPrivate *priv = mpdclient->priv;
-  MpdData               *data;
-  gchar                 *basename;
-  gint                   id;
-  gboolean               found = FALSE;
-
-  if (is_dir && strcmp (filename, ""))
-    {
-      while (xfmpc_mpdclient_playlist_read (mpdclient, &id, NULL, &basename, NULL, NULL))
-        {
-          if (g_str_has_prefix (basename, filename))
-            found = TRUE;
-
-          g_free (basename);
-        }
-
-        if (found)
-          return TRUE;
-    }
-  else
-    {
-      mpd_playlist_search_start (priv->mi, TRUE);
-      mpd_playlist_search_add_constraint (priv->mi, MPD_TAG_ITEM_FILENAME, filename);
-      data = mpd_playlist_search_commit (priv->mi);
-
-      if (data != NULL)
-        {
-          mpd_data_free (data);
-          return TRUE;
-        }
-    }
-
-  return FALSE;
-}
-
 
 
 gboolean
@@ -919,11 +933,20 @@ xfmpc_mpdclient_database_search (XfmpcMpdclient *mpdclient,
 
 
 
+void
+xfmpc_mpdclient_reload (XfmpcMpdclient *mpdclient)
+{
+  g_signal_emit (mpdclient, signals[SIG_DATABASE_CHANGED], 0);
+  g_signal_emit (mpdclient, signals[SIG_PLAYLIST_CHANGED], 0);
+}
+
+
+
 static gchar *
 _get_formatted_name (mpd_Song *song)
 {
-  XfmpcPreferences *preferences = xfmpc_preferences_get ();
-  XfmpcSongFormat song_format;
+  XfmpcPreferences *preferences = xfmpc_preferences_get_default ();
+  XfmpcPreferencesSongFormat song_format;
   gchar *format_custom;
   gchar *formatted_name;
 
@@ -936,7 +959,7 @@ _get_formatted_name (mpd_Song *song)
     {
       formatted_name = g_path_get_basename (song->file);
     }
-  else if (song_format == XFMPC_SONG_FORMAT_CUSTOM)
+  else if (song_format == XFMPC_PREFERENCES_SONG_FORMAT_CUSTOM_FORMAT)
     {
       formatted_name = _get_formatted_name_custom (song, format_custom);
     }
@@ -951,17 +974,17 @@ _get_formatted_name (mpd_Song *song)
 
 static gchar *
 _get_formatted_name_predefined (mpd_Song *song,
-                                XfmpcSongFormat song_format)
+                                XfmpcPreferencesSongFormat song_format)
 {
   gchar *formatted_name, *tmp;
 
   switch (song_format)
     {
-    case XFMPC_SONG_FORMAT_TITLE:
+    case XFMPC_PREFERENCES_SONG_FORMAT_TITLE:
       formatted_name = g_strdup_printf ("%s", song->title);
       break;
 
-    case XFMPC_SONG_FORMAT_ARTIST_TITLE:
+    case XFMPC_PREFERENCES_SONG_FORMAT_ARTIST_TITLE:
       if (NULL != song->artist)
         {
           formatted_name = g_strdup_printf ("%s - %s", song->artist, song->title);
@@ -972,7 +995,7 @@ _get_formatted_name_predefined (mpd_Song *song,
         }
       break;
 
-    case XFMPC_SONG_FORMAT_ALBUM_TITLE:
+    case XFMPC_PREFERENCES_SONG_FORMAT_ALBUM_TITLE:
       if (NULL != song->album)
         {
           formatted_name = g_strdup_printf ("%s - %s", song->album, song->title);
@@ -983,7 +1006,7 @@ _get_formatted_name_predefined (mpd_Song *song,
         }
       break;
 
-    case XFMPC_SONG_FORMAT_ARTIST_TITLE_DATE:
+    case XFMPC_PREFERENCES_SONG_FORMAT_ARTIST_TITLE_DATE:
       if (NULL != song->artist && NULL != song->date)
         {
           formatted_name = g_strdup_printf ("%s - %s (%s)", song->artist, song->title, song->date);
@@ -1002,7 +1025,7 @@ _get_formatted_name_predefined (mpd_Song *song,
         }
       break;
 
-    case XFMPC_SONG_FORMAT_ARTIST_ALBUM_TITLE:
+    case XFMPC_PREFERENCES_SONG_FORMAT_ARTIST_ALBUM_TITLE:
       if (NULL != song->artist && NULL != song->album)
         {
           formatted_name = g_strdup_printf ("%s - %s - %s", song->artist, song->album, song->title);
@@ -1021,7 +1044,7 @@ _get_formatted_name_predefined (mpd_Song *song,
         }
       break;
 
-    case XFMPC_SONG_FORMAT_ARTIST_ALBUM_TRACK_TITLE:
+    case XFMPC_PREFERENCES_SONG_FORMAT_ARTIST_ALBUM_TRACK_TITLE:
       formatted_name = g_strdup ("");
 
       if (song->artist != NULL)
