@@ -51,7 +51,7 @@ typedef struct _XfmpcMpdclientPrivate       XfmpcMpdclientPrivate;
 
 GType                   xfmpc_mpdclient_get_type                () G_GNUC_CONST;
 
-XfmpcMpdclient *        xfmpc_mpdclient_get                     ();
+XfmpcMpdclient *        xfmpc_mpdclient_get_default             ();
 gboolean                xfmpc_mpdclient_connect                 (XfmpcMpdclient *mpdclient);
 void                    xfmpc_mpdclient_disconnect              (XfmpcMpdclient *mpdclient);
 gboolean                xfmpc_mpdclient_is_connected            (XfmpcMpdclient *mpdclient);
@@ -104,9 +104,6 @@ gboolean                xfmpc_mpdclient_playlist_read           (XfmpcMpdclient
 gboolean                xfmpc_mpdclient_playlist_clear          (XfmpcMpdclient *mpdclient);
 gint                    xfmpc_mpdclient_playlist_get_length     (XfmpcMpdclient *mpdclient);
 gint                    xfmpc_mpdclient_playlist_get_total_time (XfmpcMpdclient *mpdclient);
-gboolean                xfmpc_mpdclient_playlist_has_filename   (XfmpcMpdclient *mpdclient,
-                                                                 const gchar *filename,
-                                                                 gboolean is_dir);
 
 gboolean                xfmpc_mpdclient_database_refresh        (XfmpcMpdclient *mpdclient);
 gboolean                xfmpc_mpdclient_database_read           (XfmpcMpdclient *mpdclient,
@@ -119,6 +116,8 @@ gboolean                xfmpc_mpdclient_database_search         (XfmpcMpdclient
                                                                  gchar **filename,
                                                                  gchar **basename);
 
+void                    xfmpc_mpdclient_reload                  (XfmpcMpdclient *mpdclient);
+
 XfmpcSongInfo *         xfmpc_mpdclient_get_song_info           (XfmpcMpdclient *mpdclient,
                                                                  gint id);
 
